#!/usr/bin/env bash
#
# Soubor: test.sh
# Datum:  24.04.2007
# Autor:  David Martinek, martinek@fit.vutbr.cz
# Popis:  Otestuje správnost implementace projektu Rozdíl kalendářních dat.
#
# Poznámka: Pokud v tomto skriptu některým příkazům nerozumíte, použijte 
#           manuálové a info stránky. Učebnice jazyka BASH najdete na webu,
#           např.: http://www.abclinuxu.cz/clanky/navody/bash-i
#                  http://linuxdoc.linux.cz/LDP/Bash-Beginners-Guide/html/
#                  http://linuxdoc.linux.cz/LDP/abs/html/index.html

# jméno projektu, odvíjí se od něj i jméno zdrojového souboru
PROJ=datumy

# Spustí program se zadaným parametrem a výsledek vypíše ve formátované podobě 
# na standardní výstup
# `tyto` uvozovky (na anglické klávesnici vlevo nahoře pod znakem ~) spustí text,
# který se nachází uvnitř jako příkaz a vše co program vypíše na standardní výstup
# se vrátí jako textová hodnota
function Run
{
  echo "./$PROJ <<< $1 -> "`./$PROJ <<< $1`
  
# pokud by program četl ze stdin více řádků, mohly by být testovací hodnoty
# uloženy v pomocném souboru nebo by je šlo zadat takto přímo ve skriptu:
# ./program << __ZNACKA__
# 02.01.2000-01.01.2000
# 02.01.2000-01.01.2123
# 02.01.2888-01.01.2000
# 02.01.2050-01.01.2400
# __ZNACKA__
}

# nastavení programu diff, který slouží pro porovnávání se správnými
# výsledky
DIFFPARAMS="--ignore-blank-lines --ignore-all-space"

# otestuje správnost implementace na množině vstupních hodnot
# -- přestupné roky --
function Test1
{
  (
    Run 02.01.2000-01.01.2000
    Run 01.01.2000-01.01.2000
    Run 28.02.2000-28.02.2001
    Run 29.02.2000-28.02.2001
    Run 29.02.2000-01.03.2001
    Run 01.03.2000-28.02.2001
    Run 01.03.2001-29.02.2000
    Run 17.4.1978-8.11.1620
    Run 17.4.1978-15.10.1582
    Run 29.02.2000-29.02.1600
    Run 28.02.2001-01.03.1599
    Run 01.03.2001-28.02.1599
  ) &> $PROJ.test1
  diff $DIFFPARAMS ideal.test1 $PROJ.test1
  if [ $? -gt 0 ]; then
    echo "*** NESHODUJE SE SE VZOROVÝM ŘEŠENÍM ***"
  else
    echo "*** OK ***"
  fi

}

# -- extrémní hodnoty --
function Test2
{
  ( 
    Run 31.12.11000000-15.10.1582
    Run 31.12.11000000-1.1.1
    Run 15.10.1582-31.12.11000000
    Run 17.00004.1978-7.3.24063
  ) &> $PROJ.test2
  diff $DIFFPARAMS ideal.test2 $PROJ.test2
  if [ $? -gt 0 ]; then
    echo "*** NESHODUJE SE SE VZOROVÝM ŘEŠENÍM ***"
  else
    echo "*** OK ***"
  fi 
}

# otestuje reakci programu na chybně zadaná vstupní data
function Test3
{
  echo "............. Test a. Chybný formát vstupu ................."
  echo "./$PROJ <<< 01.01.2000+02.01.2000"
  ./$PROJ <<< 01.01.2000+02.01.2000
  echo "................. Test b. Nelegální datumy ................."
  echo "./$PROJ <<< 29.02.2001-29.2.2000"
  ./$PROJ <<< 29.02.2001-29.2.2000
  echo "./$PROJ <<< 01.15.2001-31.4.2000"
  ./$PROJ <<< 01.15.2001-31.04.2000 
  echo "*** konec testu ***"
}

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# start programu
echo "Spouštím test. Výsledek najdete v souboru test.log"
# kulaté závorky znamenají vnořený shell - vše, co uvnitř nich vytisknu,
# můžu přesměrovat do souboru
(
  echo "Výsledek testovacího skriptu"
  # existuje soubor ./proj2 a je spustitelný?
  if [ -x $PROJ ]; then
    # Nastaví limit běhu programu na 15 vteřin, kdyby se něco zacyklilo,
    # bude program po této době zabit.
    ulimit -t 15
    echo "------------------ Test přestupných let --------------------"
    Test1
    echo "---------------- Test na extrémní hodnoty ------------------"
    Test2
    echo "--------------- Test reakce na chybný vstup ----------------"
    Test3
  else
    echo "*** CHYBA, ./$PROJ neexistuje! ***"
  fi
) &> test.log 
# konec souboru test.sh

